library(shiny)


#Input value for when testing the code interactively,
#list() -> input; input$n <- 30; input$pos <- 15; input$pve <- 0.4; input$predictor <- 1; input$tp <- 0.1; input$coverage <- c(5,10); input$fdr <- 0.1; input$test <- 1

# Define UI for application
shinyUI(fluidPage(
  
  
  # Application title
  titlePanel("Bisulfite Sequencing Power Simulation"),
  
  # Sidebar with a inputs for simulated data characteristics
  sidebarLayout(
    sidebarPanel(
      numericInput("n", label = "Number of samples:", value = 30), #input for number of samples
      
      selectInput("predictor", 
                  label = h3("Type of predictor"), 
                  choices = c("binary" = 1, 
                              "normally distributed" = 2, 
                              "defined by user" = 3), 
                  selected = "user"), # input type of predictor variable
      conditionalPanel(
        condition = "input.predictor == 1",
        numericInput("pos", label = "Samples under condition 1:", value = 15)  #for binary case
      ),
      conditionalPanel(
        condition = "input.predictor == 2",
        numericInput("pred_mean", label="Predictor Mean", value=0),
        numericInput("pred_sd", label="Predictor Standard Deviation", value=1) #for normally distributed case
      ),
      conditionalPanel(
        condition = "input.predictor == 3",
        fileInput("predictor2", label = ("Predictor Values")) #user inputted
      ),
      
      selectInput("test", label = h3("Type of model"), 
                  choices = list("lm" = 1, "lmm" = 2, "beta-binomial"=3, "t-test"=4), 
                  selected = 4),
      conditionalPanel(
        condition = "input.test == 3",
        checkboxInput("sure", label = ("Would you like to proceed?"))
      ),
      conditionalPanel(
        condition = "input.test == 2",
        fileInput("vcv", label = ("Covariance matrix of the structure between samples"))
      ),
      conditionalPanel(
        condition = "input.test == 2",
        sliderInput("h2", label = ("Heritability or proportion of variance explained by data structure"), min=0, max=1, step=0.01, value=0.3)
      ),
      sliderInput("coverage", label = ("Coverage range"), min = 0, 
                  max = 50, value = c(5, 10)),
      numericInput("pve", 
                  "Proportion of variance explained",
                  step=0.01, value=.3),
      numericInput("fdr", "FDR threshold:", value=.1, step=0.01),
      numericInput("tp", "Proportion of the dataset simulated as true positives", value=0.1, step=0.01),
      actionButton("goButton", "Submit")
      #submitButton("Submit")
      
    ),
    # Show a plot of the generated distribution
    mainPanel(
      h3("Q-Q plot displaying the distribution of log10(p-values) compared to those generated from a uniform distribution"),
      h4("Legend shows the percent of true positives detected at the user-defined FDR threshold and the percent of true negatives detected as false positives. For binary predictor variables, the average difference in mean DNA methylation levels between classes is also shown."),
      
      conditionalPanel(
        condition = "input.test == 3",
        h3("Warning: Running simulations with a beta binomial model is much slower than other modeling options. "),
        h4("Runtime scales with sample size: on a single core of an Intel Core i5-5200U 2.2 GHz processor, it takes ~25 minutes to simulate and analyze n=50 samples, ~80 minutes for n=100 samples, and ~4.25 hours for n=200 samples. As an alternative to running the model within the app, you may save simulated count files and test the beta-binomial model using dedicated compute resources. Code for this implementation of the beta-binomial model is available at http://www.xzlab.org/softwre/macau/bb.r.")
      ),
      
      tagList(
        tags$head(
          tags$link(rel="stylesheet", type="text/css",href="style.css"),
          tags$script(type="text/javascript", src = "busy.js")
        )
      ),
      
      div(class = "busy", checked=T,
          p("Calculation in progress...")#,
          #img(src="https://media.giphy.com/media/3oEjI6SIIHBdRxXI40/giphy.gif
          #    ")
      ),
      
      plotOutput("distPlot"),
      br(""),
      br(""),
      conditionalPanel(
        condition = "input.test == 2",
        textOutput("PosDef")
      ),
      
      #br(""),
      h3("Input Options"),
      p(strong("Type of Predictor:")," Define the predictor variable as binary, normally distributed, or user-defined. If the predictor is binary, an option appears to select the number of samples in condition 1. For a normally distributed predictor, the user selects the predictor mean and standard deviation. Finally, the user may import an n-by-1 vector of arbitrary values to use as the predictor variable."),
      p(strong("Type of Model:")," Simulated methylation levels can be modeled using a t-test (for a binary predictor), a linear model (using R's lm function), a linear mixed model (using the R package EMMREML: Akdemir & Godfrey 2015), or a beta-binomial model (available at http://www.xzlab.org/software/macau/bb.r). The beta binomial model currently takes much longer to run."),
      p(strong("Coverage Range")," indicates the possible mean coverages per site. For example, a range of 5 to 15 means that the mean coverage for each site will be drawn from a uniform distribution between 5 and 15. "),
      p(strong("Proportion of Variance Explained:")," The proportion of variance in DNA methylation levels explained by the predictor variable."), 
      p(strong("FDR Threshold:")," This allows the user to modify the false discovery rate threshold used. A false discovery rate of 0.1 corresponds to a discovery threshold that includes 10% false positives. FDRs are calculated using the q-value method of Storey and Tibshirani (Storey & Tibshirani 2003; Storey et al. 2015)."),
      p(""),
      br(""),
      p(strong("If you use this package in published work, please cite:")),
      p("Lea, A.J., Vilgalys, T.P., Durst, P.A.P., and Tung, J. Maximizing ecological and evolutionary insight from bisulfite sequencing data sets. bioRxiv. dx.doi.org/10.1101/091488 "),
      br(""),
      p(strong("References:")),
      p("Akdemir, D. and Godfrey, O.U. (2015). EMMREML: Fitting Mixed Models with Known Covariance Structures. R package version 3.1. https://CRAN.R-project.org/package=EMMREML"),
      p("Storey, J.D. & Tibshirani, R. (2003). Statistical significance for genomewide studies. Proceedings of the National Academy of Sciences: 100(16), 9440-9445."),
      p("Storey, J.D., Bass, A.B., Dabney, A., and Robinson, D. (2015). qvalue: Q-value estimation for false discovery rate control. R package version 2.2.2. http://github.com/jdstorey/qvalue")
    )
  )
))